/* SIMulador Sistema PLAnetario, M.Alvarez 2018
 * Trabajo para el TFM
 * del Master de Astrofísica de la UAM
 * 
 * Copyright (c) 2018 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of simspla
 *
 * simspla is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * simspla is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rebound.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _SIMSPLA_H_
#define _SIMSPLA_H_

#include "mrevol.h"

#define VERSION "SimSPla v1.10"

#define GOOD_RETURN 0
#define BAD_RETURN 1

#define G_KMSKG 6.67408e-20 // km^3/(kg s^2)
#define G_AUDAYKG 1.488136116290668e-34 // AU^3/(kg day^2)
#define G_AUDAYMS 2.959122082855911e-04 // AU^3/(MSun days^2)

#define AU2KM 149597870.7
#define DAY2S 86400.0
//#define MSUM2KG 1.988544e30
#define MSUM2KG 1.988475415966536e+30  // tuneado para conseguir MG = 1.327124400419394e+11

typedef struct { // componentes cartesianas 3d
    double x, y, z;
} Triplet;

/* Tipos de astros, no se usan en esta versión */

#define BT_STAR 0
#define BT_PLANET 1
#define BT_SATELLITE 2
#define BT_ASTEROID 3
#define BT_COMET 4

/* Modos de parámetros de fuerzas de marea */

#define TMODE_NOTIDES 0
#define TMODE_FIXED 1
#define TMODE_EVOL 2

typedef struct { // registro de paso
    Triplet pos;   // posición
    Triplet v;     // velocidad
    double hillr;  // radio de Hill (astro >0)
    double distm;  // distancia media al astro 0 (astro >0)
} RegStep;

typedef struct { // registro de datos de fuerzas de marea (para debug)
    double k0;
    double tl0;
    double fac0;
    double facp;
    double prefac0;
    double prefacp;
} RegTidal;

typedef struct { // astro, una estrella, planeta, satélite, etc
    char *nombre;
    int tipo;       // tipo de astro
    int estado;     // 0 si está activo, otro valor si se quitó de la simulación
    int indrb;      // correspondencia con indice de rebound
    double radio;   // radio (al inicio de la simulación)
    double masa;    // masa (al inicio de la simulación)
    double hillr;   // Radio de Hill calculado en cada paso
    int tmode;      // modo de cálculo de fuerzas de marea
    double k;       // potential Love number de grado 2 al inicio
    double tl;      // time lag al inicio
    double sig;     // parámetro de disipación, se usa si distinto de cero
    double fr;      // factor de reparto
    double presig;  // dato precalculado al inicio, (3*sig)/(2*G*k*tl*fr)
    Triplet pos;    // posición al inicio o recuperación de la simulación
    Triplet v;      // velocidad al inicio o recuperación de la simulación
    RegStep *reg;   // registro de pasos
    double acudistm;  // acumulado para calculo distancia media al astro 0 (astro >0)
} Body;

typedef struct {
    char *datadir;                // directorio donde se guardaran los datos
    int genbdata;                 // generar ficheros de datos de astros
    char bdataformat;             // T=texto, B=binario, A=ambos
    int gengdata;                 // generar ficheros de datos globales
    char gdataformat;             // T=texto, B=binario, A=ambos
    int genmindist;               // generar fichero de minimas distancias
    int gendeffiles;              // generar ficheros de definición tras cada ciclo
    char *logfile;                // fichero de mensajes de log
    int showlcon;                 // mostrar mensajes de log por consola 0=no, 1=si
} ParFsal;

typedef struct {     // registro de datos globales
    double e;          // energía
    double diffe;      // error de energía
    double elost;      // energía perdida por variación de masa del astro principal y merges
    double diffelost;  // error de energía teniendo en cuenta elost
    double ma0;        // masa del astro principal
    double ra0;        // radio del astro principal
    RegTidal rt;       // datos fuerza de marea planeta n (para debug)
    Triplet L;         // momento angular
} RegDGlobal;

typedef struct {  // registro de distancias
    double d;       // última distancia calculada
    double dmin;    // distancia mínima
    double tmin;    // t en que ocurrió mínima distancia
    double dlim;    // distancia límite para considerar encuentro cercano
    int aec;        // 1 si se emitió ya un aviso de encuentro cercano
} RegDist;

/* Tipos de unidades de medida */

#define SU_KMSKG    0 // km, s, kg
#define SU_AUDAYKG  1 // AU, day, kg
#define SU_AUDAYMS  2 // AU, day, MSun

/* Tipos de integradores */

#define INTG_WHFAST 0
#define INTG_DKD    1

typedef struct { // Sistema planetario, typedef System
    char *nombre;
    int units;         // sistema de unidades
    int integrator;    // integrador
    double G;          // valor de G para el sistema de unidades
    double stepdt;     // dt de la simulacion
    long nsteps;       // pasos de un ciclo
    long nsave;        // cada cuantos pasos se guarda la info
    long showpro;      // mostrar progreso cada nsave*showpro pasos
    long countsp;      // cuenta de mostrar progreso
    long nregs;        // nº de registros (nsteps / nsave)
    long ncycles;      // numero de ciclos de la simulacion
    ParFsal pfsal;     // parámetros ficheros de salida
    int nbodies;       // número de astros
    int nbcforce;      // número de astros a considerar en el calculo de fuerzas
    Body *astro;       // array de astros
    RegDGlobal *rege;  // registros de datos globales
    long nfcycle;      // numero del primer ciclo (!= 0 en restaurar)
    double t0;         // t inicial del ciclo 0 (!= 0 en restaurar)
    double eini;       // valor de la energía al comiezo de la simulación (!= 0 en restaurar)
    double elost_acu;  // energía acumulada por perdida de masa o merges (!= 0 en restaurar)
    double m0_acu;     // masa acumulada al astro principal por merges (!= 0 en restaurar)
    double mul_rh;     // multiplicador radios Hill para detectar encuentros cercanos
    int mre_active;    // true si se usa la evolución de radio y masa
    char *mre_fname;   // nombre fichero evolución radio y masa cuerpo principal
    double mre_t0;     // correspondencia T0 en t de la evolución con t en inicio simulacion
    double mre_utsim;  // correspondencia en años de la unidad de tiempo de la simulación
    long mre_evstep;   // intervalo de pasos en que se actualiza radio y masa
    MREvolution *mre;  // estructura de la evolución de radio y masa
    int nreg_dist;     // número de registros de distancia (nbodies-1)+(nbodies-2)+...+1
    RegDist *rdist;    // registros de distancia
    double tideson;    // minima distancia para considerar fuerzas de marea
    RegTidal rt;       // datos fuerza de marea planeta n (para debug)
} System;

/* funciones en inicio.c */

System * genera_system_from_filedef(char *fname);
int guarda_system_to_filedef(System *s, long ncycle);
void destruye_system(System *s);
int find_body_by_name(System *s, char *name);

/* funciones en log.c */

int openlog(char *fname, int pstdio);
void closelog(void);
void putslog(const char *text);
void printflog(const char *format, ...);
void flushlog(void);

/* funciones en imprime.c */

void imprime_system(System *s);
int guarda_datos_astro(System *s, int nbody, long ncycle);
int guarda_datos_globales(System *s, long ncycle);
int guarda_datos_mind(System *s, long ncycle);

#endif /* _SIMSPLA_H_ */
